import json
import time
import serial
import pynmea2
import re
import paho.mqtt.client as mqtt
import time

HOST = "mqtt.waveshare.cloud"
PORT = 1883
Client_ID = ""  # 填入client id
Publish_Topic = ''  # 填入上报主题
Subscribe_Topic = ''  # 填入订阅主题

client = mqtt.Client(Client_ID)


def on_connect(client, userdata, flags, rc):
    print("Connected with result code " + str(rc))
    if rc == 0:
        client.subscribe(Subscribe_Topic)


def on_message(client, userdata, msg):
    print("Received message: " + msg.payload.decode())


def define():
    client.on_connect = on_connect
    client.on_message = on_message
    client.connect(HOST, PORT, 60)
    client.loop_start()


class AT:
    def __init__(self, port='/dev/ttyTHS1', baud_rate=115200):
        self.port = port
        self.baud_rate = baud_rate
        self.ser = None
        self.initialize_gnss()

    def initialize_gnss(self):
        try:
            self.ser = serial.Serial(self.port, self.baud_rate, timeout=1)
            self.send_at_command('AT+CGNSSPORTSWITCH=0,0')
            self.send_at_command('AT+CGNSSTST=0')
            self.send_at_command('AT+CGNSSPWR=0')
        except serial.SerialException as e:
            raise Exception(f"Error initializing GNSS: {e}")

    def open_serial_connection(self):
        try:
            self.ser = serial.Serial(self.port, self.baud_rate, timeout=1)
            return self.send_at_command('AT') == "OK"
        except serial.SerialException as e:
            raise Exception(f"Error opening serial connection: {e}")

    def send_at_command(self, command, wait_for_response=True):
        try:
            encoded_command = command.encode('utf-8') + b'\r\n'
            self.ser.write(encoded_command)
            print(f"Sent command: {command}")

            if wait_for_response:
                response = self.ser.readlines()[1].decode('utf-8').strip()
                print(f"Response: {response}")
                return response
            else:
                return None
        except serial.SerialException as e:
            raise Exception(f"Error sending AT command: {e}")

    def gnss_pwr_open(self, enable):
        command = f'AT+CGNSSPWR={int(enable)}'
        return self.send_at_command(command) == "OK"

    def gnss_layout(self):
        return self.send_at_command('AT+CGNSSTST=1') == "OK"

    def gnss_layout_switch(self):
        time.sleep(10)
        self.send_at_command('AT+CGNSSPORTSWITCH=0,1', wait_for_response=False)
        return True

    def gnss_params(self):
        try:
            response = str(self.ser.readline(), encoding='utf-8')
            if response.startswith("$GNRMC"):
                rmc = pynmea2.parse(response)
                return rmc
            else:
                return None
        except serial.SerialException as e:
            raise Exception(f"Error reading GNSS params: {e}")


if __name__ == '__main__':
    define()
    try:
        at_instance = AT()
        if at_instance.open_serial_connection():
            at_instance.gnss_pwr_open(1)
            at_instance.gnss_layout()
            at_instance.gnss_layout_switch()
            while True:
                response = at_instance.gnss_params()
                if response:
                    if re.match("^\d+?\.\d+?$", response.lat) is not None:
                        print(f"Latitude: {response.latitude}, Longitude: {response.longitude}")
                        Msgsend = {"data": {
                            "Latitude": response.latitude,
                            "Longitude": response.longitude
                        }}
                        client.publish(Publish_Topic, json.dumps(Msgsend))
                    else:
                        print(response)
    except KeyboardInterrupt:
        print("KeyboardInterrupt received. Cleaning up before exiting.")
        AT().initialize_gnss()
        AT().ser.close()
        client.loop_stop()
        client.disconnect()
        print("MQTT client disconnected.")
    except Exception as e:
        print(f"An error occurred: {e}")
    finally:
        print("Program terminated.")
