import json
import re
import threading
# import RPi.GPIO as GPIO
# from gpiozero import LED

import serial
import time
import os

ID = ""
sub = ""
pub = ""

mqtt_server = "mqtt.waveshare.cloud"

def powerLed(swatch):
    led = open('/sys/class/leds/pwr/brightness', 'w', 1)
    led.write(str(swatch))
    led.close()


def getLed():
    led = open('/sys/class/leds/pwr/brightness', 'r', 1)
    state = led.read()
    led.close()
    return state


class AT:
    def __init__(self, port='/dev/ttyTHS1', baud_rate=115200):
        self.port = port
        self.baud_rate = baud_rate
        self.ser = serial.Serial(self.port, self.baud_rate, timeout=1)
        self.destory_mqtt_init()

    def open_serial_connection(self):
        try:
            return self.send_at_command('AT', True)[1].decode('utf-8').strip() == "OK"
        except serial.SerialException as e:
            raise Exception(f"Error opening serial connection: {e}")

    def destory_mqtt_init(self):
        try:
            self.send_at_command('AT+CMQTTDISC=0,120', True)
            self.send_at_command('AT+CMQTTREL=0', True)
            self.send_at_command('AT+CMQTTSTOP', True)
        except serial.SerialException as e:
            raise Exception(f"Error initializing MQTT: {e}")

    def send_at_command(self, command, wait_for_response=False):
        try:
            encoded_command = command.encode('utf-8') + b'\r\n'
            self.ser.write(encoded_command)
            if wait_for_response:
                print(f"Sent command: {command}")
                response = self.ser.readlines()
                print(f"Response: {response[1].decode('utf-8').strip()}")
                return response
            else:
                time.sleep(0.5)
        except serial.SerialException as e:
            raise Exception(f"Error sending AT command: {e}")

    def init_mqtt(self):
        self.send_at_command('AT+CMQTTSTART', True)  # START MQTT
        self.send_at_command(f'AT+CMQTTACCQ=0,"{ID}",0', True)
        res = self.send_at_command(f'AT+CMQTTCONNECT=0,"tcp://{mqtt_server}",20,1', True)[3].decode('utf-8').strip()
        status = int(res.split("+CMQTTCONNECT: 0,")[1])
        if status != 0:
            raise Exception(f"Error MQTT Response:{status}")
        else:
            print(f"MQTT Server: {mqtt_server} Connect Successful!")
            return True

    def publish_message(self, message):
        self.send_at_command(f'AT+CMQTTTOPIC=0,{len(pub)}')
        self.send_at_command(f'{pub}')
        self.send_at_command(f'AT+CMQTTPAYLOAD=0,{len(message)}')
        self.send_at_command(f'{message}')
        self.send_at_command('AT+CMQTTPUB=0,0,60')

    def subscribe_message(self):
        self.send_at_command(f'AT+CMQTTSUB=0,{len(sub)},1', True)
        self.send_at_command(f'{sub}', True)

    def subscribe_messages(self):
        while True:
            response = self.ser.readline().decode("utf-8")
            # print(response,end='')
            if response.startswith('+CMQTTRXPAYLOAD:'):
                next_line = self.ser.readline().decode('utf-8')
                print("Subscribe Message:", json.loads(next_line), end='')

    def start_subscribe_thread(self):
        self.thread_response_stop_flag = threading.Event()
        thread_response = threading.Thread(target=self.subscribe_messages)
        thread_response.daemon = True
        thread_response.start()

    def destroy(self):
        if hasattr(self, 'thread_response'):
            print("Destroying thread response...")
            self.thread_response_stop_flag.set()
        self.destory_mqtt_init()
        self.ser.close()


if __name__ == '__main__':
    try:
        at_instance = AT()
        if at_instance.open_serial_connection():
            print('Opened serial connection')
            if at_instance.init_mqtt():
                at_instance.subscribe_message()
                at_instance.start_subscribe_thread()
                while True:
                    time.sleep(5)
                    power_stats = int(getLed())
                    if (power_stats == 0):
                        power_LED = 0
                    else:
                        power_LED = 1
                    updateMsn = {
                        'data':
                            {
                                'PowerLed': power_LED
                            }
                    }
                    JsonUpdataMsn = json.dumps(updateMsn)
                    at_instance.publish_message(JsonUpdataMsn)
                    print(JsonUpdataMsn)

    except KeyboardInterrupt:
        print("KeyboardInterrupt received. Cleaning up before exiting.")
        AT().destroy()
    except Exception as e:
        print(f"An error occurred: {e}")
    finally:
        print("Program terminated.")
